@@ -46,7 +46,6 @@ from execcommand import *
 from sendmessage import *
 from about_dialog import About
 from client_information import ClientInformation
-from remote_assistance import RemoteAssistance
 from epoptes.daemon import uiconnection
 from epoptes.core.lib_users import *
 from epoptes.common import ltsconf
@@ -69,7 +68,8 @@ class EpoptesGui(object):
             """ip -oneline -family inet link show | sed -n '/.*ether[[:space:]]*\\([[:xdigit:]:]*\).*/{s//\\1/;y/abcdef-/ABCDEF:/;p;}'
             echo $LTSP_CLIENT_MAC"""],
             stdout=subprocess.PIPE).communicate()[0].split()
-        if os.getuid() != 0:
+        self.uid = os.getuid()
+        if self.uid != 0:
             if 'thumbnails_width' in config.user:
                 self.scrWidth = config.user['thumbnails_width']
             if 'thumbnails_height' in config.user:
@@ -79,14 +79,24 @@ class EpoptesGui(object):
         self.fat = gtk.gdk.pixbuf_new_from_file('images/fat.svg')
         self.standalone = gtk.gdk.pixbuf_new_from_file('images/standalone.svg')
         self.imagetypes = {'thin' : self.thin, 'fat' : self.fat,
-            'standalone' : self.standalone, 'server' : self.standalone, 'offline' : self.offline}
+            'standalone' : self.standalone, 'offline' : self.offline}
         
         self.wTree = gtk.Builder()
         self.wTree.add_from_file('epoptes.ui')
         
+        self.get = lambda obj: self.wTree.get_object(obj)
+        self.showRealNames = False
+        if config.settings.has_option('GUI', 'showRealNames'):
+            self.get('mcShowRealNames').set_active(config.settings.getboolean('GUI', 'showRealNames'))
+        
         # Connect glade handlers with the callback functions
         self.wTree.connect_signals(self)
-        self.get = lambda obj: self.wTree.get_object(obj)
+        
+        
+        # Hide the remote assistance menuitem if epoptes-client is not installed
+        if not os.path.isfile('/usr/share/epoptes-client/remote-assistance'):
+            self.get('mi_remote_assistance').set_property('visible', False)
+            self.get('remote_assistance_separator').set_property('visible', False)
         
         self.gstore = gtk.ListStore(str, object, bool)
         
@@ -132,9 +142,7 @@ class EpoptesGui(object):
                 self.get(config.settings.get('GUI', 'label')).set_active(True)
             except:
                 pass
-        self.showRealNames = False
-        if config.settings.has_option('GUI', 'showRealNames'):
-            self.get('mcShowRealNames').set_active(config.settings.getboolean('GUI', 'showRealNames'))
+        
 
     #################################################################
     #                       Callback functions                      #
@@ -177,7 +185,7 @@ class EpoptesGui(object):
         
         settings.set('GUI', 'selected_group', sel_group)
         settings.set('GUI', 'showRealNames', self.showRealNames)
-        f = open(os.path.expanduser('~/.config/epoptes/settings'), 'wb')
+        f = open(os.path.expanduser('~/.config/epoptes/settings'), 'w')
         settings.write(f)
         f.close()
         
@@ -205,29 +213,30 @@ class EpoptesGui(object):
 
     def poweroff(self, widget):
         """Shut down the selected clients."""
-        self.execOnSelectedClients("shutdown", root="auto",
+        self.execOnSelectedClients(["shutdown"],
             warn=_('Are you sure you want to shutdown all the computers?'))
 
     def reboot(self, widget):
         """Reboot the selected clients."""
-        # FIXME: (Not) waiting on purpose to cause some delay to avoid 
-        # any power strain.
-        self.execOnSelectedClients("reboot", root="auto",
+        self.execOnSelectedClients(["reboot"],
             warn=_('Are you sure you want to reboot all the computers?'))
 
+
     def logout(self, widget):
         """Log off the users of the selected clients."""
-        self.execOnSelectedClients("logout",
+        self.execOnSelectedClients(['logout'], mode=EM_SESSION,
             warn=_('Are you sure you want to log off all the users?'))
 
 
     def reverseConnection(self, widget, path, view_column, cmd):
         # Open vncviewer in listen mode
         if self.vncviewer is None:
-            self.vncviewer = subprocess.Popen(['xvnc4viewer', '-listen'])
+            self.vncport = self.findUnusedPort()
+            self.vncviewer = subprocess.Popen(
+                ['xvnc4viewer', '-listen', str(self.vncport)])
 
         # And, tell the clients to connect to the server
-        self.execOnSelectedClients(cmd)
+        self.execOnSelectedClients([cmd, self.vncport])
 
 
     def assistUser(self, widget, path=None, view_column=None):
@@ -238,49 +247,50 @@ class EpoptesGui(object):
         self.reverseConnection(widget, path, view_column, 'get_monitored')
 
 
-    def findUnusedPort(self, base=None):
-        """Find an unused port, optionally starting from "base"."""
+    def findUnusedPort(self):
+        """Find an unused port."""
         import socket
 
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        if base is None:
-            s.bind(('', 0))
-            s.listen(1)
-            port = s.getsockname()[1]
-            s.close()
-            return port
-        else:
-            port = int(base)
-            while port < 65535:
-                try:
-                    s.connect(('', port))
-                    s.shutdown(2)
-                    port += 1
-                except:
-                    return port
-            return None
+        s.bind(('', 0))
+        s.listen(1)
+        port = s.getsockname()[1]
+        s.close()
+        return port
 
 
     def _broadcastScreen(self, fullscreen=''):
         if self.vncserver is None:
+            import random
+            import string
+            pwdfile=os.path.expanduser('~/.config/epoptes/vncpasswd')
+            pwd=''.join(random.sample(string.letters + string.digits, 8))
+            subprocess.call(['x11vnc', '-storepasswd', pwd, pwdfile])
+            f=open(pwdfile)
+            pwd=f.read()
+            f.close()
+            self.pwd=''.join('\\%o' % ord(c) for c in pwd)
             self.vncport = self.findUnusedPort()
-            # TODO: use a password instead of -allow
             self.vncserver = subprocess.Popen(['x11vnc', '-noshm', '-nopw',
                 '-quiet', '-viewonly', '-shared', '-forever', '-nolookup',
-                '-24to32', '-rfbport', str(self.vncport), '-allow',
-                '127.,192.168.,10.,169.254.' ])
-        self.execOnSelectedClients('stop_screensaver')
-        self.execOnSelectedClients('receive_broadcast %d %s' % (self.vncport,
-            fullscreen), root=True)
-    
+                '-24to32', '-rfbport', str(self.vncport), '-rfbauth', pwdfile])
+        self.execOnSelectedClients(['stop_screensaver'],
+            mode=EM_SYSTEM_AND_SESSION)
+        self.execOnSelectedClients(["receive_broadcast", self.vncport, 
+                                    self.pwd, fullscreen], mode=EM_SYSTEM_OR_SESSION)
+
+
     def broadcastScreen(self, widget):
         self._broadcastScreen('true')
-        
+
+
     def broadcastScreenWindowed(self, widget):
         self._broadcastScreen('')
 
+
     def stopTransmissions(self, widget):
-        self.execOnClients('stop_transmissions', self.cstore, None, True)
+        self.execOnClients(['stop_receptions'], self.cstore,
+            mode=EM_SYSTEM_AND_SESSION)
         if not self.vncserver is None:
             self.vncserver.kill()
             self.vncserver = None
@@ -292,20 +302,23 @@ class EpoptesGui(object):
         # If Cancel or Close were clicked
         if cmd == '':
             return
-        as_root = False
+        em = EM_SESSION
         if cmd[:5] == 'sudo ':
-            as_root = True
+            em = EM_SYSTEM
             cmd = cmd[4:]
-        self.execOnSelectedClients('execute ' + cmd, root=as_root)
+        self.execOnSelectedClients(['execute', cmd], mode=em)
+
 
-    ## FIXME FIXME: Don't use zenity, use the message command instead...
     def sendMessageDialog(self, widget):
-        cmd = startSendMessageDlg()
-        if cmd != "": # Command is 'valid', execute on selected clients 
-            self.execOnSelectedClients('execute ' + cmd)
-    
+        msg = startSendMessageDlg()
+        if msg:
+            cmd = list(msg)
+            cmd.insert(0, 'message')
+            self.execOnSelectedClients(cmd)
+
+
     ## FIXME / FIXUS: Should we allow it?
-    def openTerminal(self, as_root):
+    def openTerminal(self, em):
         clients = self.getSelectedClients()
         
         # If there is no client selected, send the command to all
@@ -321,44 +334,51 @@ class EpoptesGui(object):
 
             subprocess.Popen(['xterm', '-e', 'socat',
                 'tcp-listen:%d,keepalive=1' % port, 'stdio,raw,echo=0'])
-            self.execOnClients('remote_term %d' % port, [client],
-                root=as_root)
+            self.execOnClients(['remote_term', port], [client], mode=em)
+
 
     def openUserTerminal(self, widget):
-        self.openTerminal(False)
+        self.openTerminal(EM_SESSION)
+
 
     def openRootTerminal(self, widget):
-        self.openTerminal(True)
+        self.openTerminal(EM_SYSTEM)
+
 
     def remoteRootTerminal(self, widget):
-        self.execOnSelectedClients('root_term', root=True)
+        self.execOnSelectedClients(['root_term'], mode=EM_SYSTEM)
     ## END_FIXUS
 
+
     def lockScreen(self, widget):
         """
         Lock screen for all the selected clients, displaying a message
         """
         msg = _("The screen is locked by a system administrator.")
-        self.execOnSelectedClients('lock_screen 0 %s' % pipes.quote(msg))
+        self.execOnSelectedClients(['lock_screen', 0, msg])
+
 
     def unlockScreen(self, widget):
         """
         Unlock screen for all clients selected
         """
-        self.execOnSelectedClients('unlock_screen')
+        self.execOnSelectedClients(['unlock_screen'], mode=EM_SESSION_AND_SYSTEM)
+
 
     def soundOff(self, widget):
         """
         Disable sound usage for clients selected
         """
-        self.execOnSelectedClients('mute 0', root=True)
+        self.execOnSelectedClients(['mute_sound', 0], mode=EM_SYSTEM_OR_SESSION)
+
 
     def soundOn(self, widget):
         """
         Enable sound usage for clients selected
         """
-        self.execOnSelectedClients('unmute', root=True)
-    
+        self.execOnSelectedClients(['unmute_sound'], mode=EM_SYSTEM_AND_SESSION)
+
+
     def on_remove_from_group_clicked(self, widget):
         clients = self.getSelectedClients()
         group = self.getSelectedGroup()[1]
@@ -367,79 +387,96 @@ class EpoptesGui(object):
             for client in clients:
                 group.remove_client(client[C_INSTANCE])
             self.fillIconView(self.getSelectedGroup()[1])
-    
+
+
     def set_move_group_sensitivity(self):
         selected = self.getSelectedGroup()
         selected_path = self.gstore.get_path(selected[0])[0]
         blocker = not selected[1] is self.default_group
         self.get('move_group_up').set_sensitive(blocker and selected_path > 1)
         self.get('move_group_down').set_sensitive(blocker and selected_path < len(self.gstore)-1)
-    
+
+
     def on_move_group_down_clicked(self, widget):
         selected_group_iter = self.getSelectedGroup()[0]
         self.gstore.swap(selected_group_iter, self.gstore.iter_next(selected_group_iter))
         self.set_move_group_sensitivity()
-    
+
+
     def on_move_group_up_clicked(self, widget):
         selected_group_iter = self.getSelectedGroup()[0]
         previous_iter = self.gstore.get_iter(self.gstore.get_path(selected_group_iter)[0]-1)
         self.gstore.swap(selected_group_iter, previous_iter)
         self.set_move_group_sensitivity()
-        
+
+
     def on_remove_group_clicked(self, widget):
         group_iter = self.getSelectedGroup()[0]
         group = self.gstore[group_iter][G_INSTANCE]
         
         if self.warnDlgPopup(_('Are you sure you want to remove group "%s"?') % group.name):
             self.gstore.remove(group_iter)
-            
+
+
     def on_add_group_clicked(self, widget):
         new_group = structs.Group()
         iter = self.gstore.append([new_group.name, new_group, True])
         # Edit the name of the newly created group
         self.gtree.set_cursor(self.gstore.get_path(iter), self.get('group_name_column'), True)
-    
+
+
     def on_group_renamed(self, widget, path, new_name):
         self.gstore[path][G_LABEL] = new_name
         self.gstore[path][G_INSTANCE].name = new_name
-        
+
+
     #FIXME: Remove the second parameter, find a better way
     def on_tb_client_properties_clicked(self, widget=None):
         ClientInformation(self.getSelectedClients(), self.daemon.command).run()
         self.setLabel(self.getSelectedClients()[0])
-    
+
+
     def on_mi_remote_assistance_activate(self, widget=None):
-        RemoteAssistance().run()
-    
+        path = '/usr/share/epoptes-client'
+        subprocess.Popen('%s/remote-assistance' %path, shell=True, cwd=path)
+
+
     def on_mi_about_activate(self, widget=None):
         About().run()
-        
+
+
     def on_cViewHU_toggled(self, mitem):
         self.set_cView(1, 0)
         config.settings.set('GUI', 'label', 'miClientsViewHostUser')
-    
+
+
     def on_cViewUH_toggled(self, mitem):
         self.set_cView(0, 1)
         config.settings.set('GUI', 'label', 'miClientsViewUserHost')
-    
+
+
     def on_cViewH_toggled(self, mitem):
         self.set_cView(-1, 0)
         config.settings.set('GUI', 'label', 'miClientsViewHost')
-    
+
+
     def on_cViewU_toggled(self, mitem):
         self.set_cView(0, -1)
         config.settings.set('GUI', 'label', 'miClientsViewUser')
-    
+
+
     def set_cView(self, user_pos= -1, name_pos=0):
         # Save the order so all new clients get the selected format
         self.cView_order = (user_pos, name_pos)
         for row in self.cstore:            
             self.setLabel(row)
-    
+
+
     def connected(self, daemon):
         self.daemon = daemon
         daemon.enumerateClients().addCallback(lambda h: self.amp_gotClients(h))
 
+
     # AMP callbacks
     def amp_clientConnected(self, handle):
         print "New connection from", handle
@@ -447,6 +484,7 @@ class EpoptesGui(object):
         d.addCallback(lambda r: self.addClient(handle, r))
         d.addErrback(lambda err: self.printErrors("when connecting client %s: %s" %(handle, err)))
 
+
     def amp_clientDisconnected(self, handle):
         print "Disconnect from", handle
         
@@ -476,17 +514,20 @@ class EpoptesGui(object):
                 if row[C_INSTANCE] is client: 
                     self.fillIconView(self.getSelectedGroup()[1])
                     break
-    
+
+
     def amp_gotClients(self, handles):
         print "Got clients:", ', '.join(handles) or 'None'
         for handle in handles:
             d = self.daemon.command(handle, u'info')
             d.addCallback(lambda r, h=handle: self.addClient(h, r, True))
             d.addErrback(lambda err: self.printErrors("when enumerating client %s: %s" %(handle, err)))
-    
+
+
     def on_button_close_clicked(self, widget):
         self.get('warningDialog').hide()
-    
+
+
     def on_group_selection_changed(self, treeselection):
         self.cstore.clear()
         selected = self.getSelectedGroup()
@@ -499,7 +540,8 @@ class EpoptesGui(object):
             self.gtree.get_selection().select_path(self.default_group.ref.get_path())
         self.get('remove_group').set_sensitive(not self.isDefaultGroupSelected())
         self.set_move_group_sensitivity()
-    
+
+
     def addToIconView(self, client):
         """Properly add a Client class instance to the clients iconview."""
         # If there are one or more users on client, add a new iconview entry
@@ -514,7 +556,8 @@ class EpoptesGui(object):
                 self.askScreenshot(hsession, True)
         else:
             self.cstore.append([self.calculateLabel(client), self.imagetypes[client.type], client, ''])
-    
+
+
     def fillIconView(self, group):
         """Fill the clients iconview from a Group class instance."""
         self.cstore.clear()
@@ -525,11 +568,13 @@ class EpoptesGui(object):
         # Add the new clients to the iconview
         for client in clients_list:
             self.addToIconView(client)
-    
+
+
     def isDefaultGroupSelected(self):
         """Return True if the default group is selected"""
         return self.getSelectedGroup()[1] is self.default_group
-    
+
+
     def getSelectedGroup(self):
         """Return a 2-tuple containing the iter and the instance
         for the currently selected group."""
@@ -538,7 +583,8 @@ class EpoptesGui(object):
             return (iter, self.gstore[iter][G_INSTANCE])
         else:
             return None
-        
+
+
     def addClient(self, handle, r, already=False):
         # already is True if the client was started before epoptes
         print "---\n**addClient's been called for", handle
@@ -548,8 +594,8 @@ class EpoptesGui(object):
                 key, value = line.split('=', 1)
                 info[key.strip()] = value.strip()
             user, host, ip, mac, type, uid, version, name = \
-             info['user'], info['hostname'], info['ip'], \
-             info['mac'], info['type'], info['uid'], info['version'], info['name']
+                info['user'], info['hostname'], info['ip'], info['mac'], \
+                info['type'], int(info['uid']), info['version'], info['name']
         except:
             print "Can't extract client information, won't add this client"
             return
@@ -558,7 +604,7 @@ class EpoptesGui(object):
         # epoptes is running, so we don't add it to the list.
         # FIXME FiXME: Both ifs don't work for root clients that run in the same
         # computer as epoptes
-        if mac in self.current_macs:
+        if (mac in self.current_macs) and (uid == self.uid):
             print "* Won't add this client to my lists"
             return False
         
@@ -570,9 +616,9 @@ class EpoptesGui(object):
                 self.shownCompatibilityWarning = True
                 dlg = self.get('warningDialog')
                 # Show different messages for LTSP clients and standalones.
-                msg = _("There was a try to connect an epoptes-client with version %s \
+                msg = _("A connection attempt was made by a client with version %s, \
 which is incompatible with the current epoptes version.\
-\n\nYou should update your chroot.") % version
+\n\nYou need to update your clients to the latest epoptes-client version.") % version
                 dlg.set_property('text', msg)
                 dlg.set_transient_for(self.mainwin)
                 dlg.show()
@@ -592,7 +638,7 @@ which is incompatible with the current epoptes version.\
             
         # Update/fill the client information
         client.type, client.hostname = type, host
-        if int(uid) == 0:
+        if uid == 0:
             # This is a root epoptes-client
             print '* I am a root client'
             client.hsystem = handle
@@ -605,7 +651,8 @@ which is incompatible with the current epoptes version.\
         
         if sel_group.has_client(client) or self.isDefaultGroupSelected():
             self.fillIconView(sel_group)
-    
+
+
     def setLabel(self, row):
         inst = row[C_INSTANCE]
         if row[C_SESSION_HANDLE]:            
@@ -616,7 +663,8 @@ which is incompatible with the current epoptes version.\
         else:
             user = ''
         row[C_LABEL] = self.calculateLabel(inst, user)
-    
+
+
     def calculateLabel(self, client, username=''):
         """Return the iconview label from a hostname/alias
         and a username, according to the user options.
@@ -636,19 +684,23 @@ which is incompatible with the current epoptes version.\
                 if user_pos == 1:
                     label += " (%s)" % username
             return label
-    
+
+
     def getShowRealNames(self):
         return self.get('')
-    
+
+
     def getAllScreenshots(self):
         # TODO: Ask for screenshots for every client (Look diff at Rev:326)
         pass
 
+
     def screenshotTimeout(self, handle):
         print "Screenshot for client %s timed out. Requesting a new one..." % handle
         self.askScreenshot(handle)
         return False
-        
+
+
     def askScreenshot(self, handle, firstTime=False):
         # Should always return False to prevent glib from calling us again
         if firstTime:
@@ -668,9 +720,8 @@ which is incompatible with the current epoptes version.\
         for client in self.cstore:
             if handle == client[C_SESSION_HANDLE]:
                 timeoutID = gobject.timeout_add(10000, lambda h=handle: self.screenshotTimeout(h))
-                self.execOnClients('screenshot %d %d'
-                    % (self.scrWidth, self.scrHeight), handles=[handle],
-                    reply=self.gotScreenshot, params=[timeoutID])
+                self.execOnClients(['screenshot', self.scrWidth, self.scrHeight], handles=[handle], 
+                                   reply=self.gotScreenshot, params=[timeoutID])
                 return False
         # That handle is no longer in the cstore, remove it
         try: del self.currentScreenshots[handle]
@@ -706,39 +757,48 @@ which is incompatible with the current epoptes version.\
         # That handle is no longer in the cstore, remove it
         try: del self.currentScreenshots[handle]
         except: pass
-    
+
+
     def getSelectedClients(self):
         selected = self.cview.get_selected_items()
         items = []
         for i in selected:
             items.append(self.cstore[i])
         return items
-        
+
+
     def changeHostname(self, mac, new_name):
         pass #FIXME: Implement this (virtual hostname)
-    
+
+
     def openLink(self, link):
         subprocess.Popen(["xdg-open", link])
 
+
     def openHelpLink(self, widget):
         self.openLink("http://www.epoptes.org")
 
+
     def openBugsLink(self, widget):
         self.openLink("https://bugs.launchpad.net/epoptes")
 
+
     def openQuestionsLink(self, widget):
         self.openLink("https://answers.launchpad.net/epoptes")
 
+
     def openTranslationsLink(self, widget):
         self.openLink("http://www.epoptes.org/translations")
 
+
     def openIRCLink(self, widget):
         user = os.getenv("USER")
         if user is None:
             user = "epoptes_user." # The dot is converted to a random digit
         self.openLink("http://webchat.freenode.net/?nick=" + user + 
             "&channels=ltsp&prompt=1")
-    
+
+
     ## FIXME: We don't use this (we want to). there was a problem with twisted :-\
     def iconsSizeScaleChanged(self, widget):
         adj = self.get('iconsSizeAdjustment')
@@ -746,11 +806,13 @@ which is incompatible with the current epoptes version.\
         self.scrHeight = int(3 * self.scrWidth / 4) # Κeep the 4:3 aspect ratio
         self.getAllScreenshots()
 
+
     def scrIncreaseSize(self, widget):
         # Increase the size of screenshots by 2 pixels in width
         adj = self.get('iconsSizeAdjustment')
         adj.set_value(adj.get_value() + 2)
 
+
     def scrDecreaseSize(self, widget):
         # Decrease the size of screenshots by 2 pixels in width
         adj = self.get('iconsSizeAdjustment')
@@ -758,6 +820,7 @@ which is incompatible with the current epoptes version.\
     
     ## END_FIXME
 
+
     def contextMenuPopup(self, widget, event):
         clicked = widget.get_path_at_pos(int(event.x), int(event.y))
 
@@ -785,6 +848,7 @@ which is incompatible with the current epoptes version.\
             menu.show()
             return True
 
+
     def on_clients_selection_changed(self, widget=None):
         selected = self.getSelectedClients()
         sensitive = False
@@ -798,16 +862,19 @@ which is incompatible with the current epoptes version.\
         else:
             self.get('miRemoveFromGroup').set_sensitive(False)
 
-    
-    ## FIXME / FIXUS: Proofread this (root etc...)
-    def execOnClients(self, command, clients=[], reply=None, root=False,
-                        handles=[], warning='', params=None):
+
+    def execOnClients(self, command, clients=[], reply=None,
+            mode=EM_SESSION_OR_SYSTEM, handles=[], warning='', params=None):
         '''reply should be a method in which the result will be sent'''
         if params is None:
             params = []
         if len(self.cstore) == 0:
             # print 'No clients'
             return False
+        
+        if isinstance(command, list) and len(command) > 0:
+            command = '%s %s' %(command[0], ' '.join([pipes.quote(str(x)) for x in command[1:]]))
+                    
         if (clients != [] or handles != []) and warning != '':
             if self.warnDlgPopup(warning) == False:
                 return
@@ -819,26 +886,35 @@ which is incompatible with the current epoptes version.\
                     cmd.addErrback(lambda err: self.printErrors("when executing command %s on client %s: %s" %(command,handle, err)))
 
         for client in clients:
-            if (root == "auto" or not root) and client[C_SESSION_HANDLE] != '':
-                handle = client[C_SESSION_HANDLE]
-            elif root and client[C_INSTANCE].hsystem != '':
-                handle = client[C_INSTANCE].hsystem
-            else:
-                continue
-            cmd = self.daemon.command(handle, unicode(command))
-            if reply:
-                cmd.addCallback(lambda re, h=handle, p=params: reply(h, re, *p))
-                cmd.addErrback(lambda err: self.printErrors("when executing command %s on client %s: %s" %(command,handle, err)))
+            sent = False
+            for em in mode:
+                if em == EM_SESSION_ONLY:
+                    handle = client[C_SESSION_HANDLE]
+                elif em == EM_SYSTEM_ONLY:
+                    handle = client[C_INSTANCE].hsystem
+                else: # em == EM_EXIT_IF_SENT
+                    if sent:
+                        break
+                    else:
+                        continue
+                if handle == '':
+                    continue
+                sent = True
+                cmd = self.daemon.command(handle, unicode(command))
+                if reply:
+                    cmd.addCallback(lambda re, h=handle, p=params: reply(h, re, *p))
+                    cmd.addErrback(lambda err: self.printErrors("when executing command %s on client %s: %s" %(command,handle, err)))
 
-    def execOnSelectedClients(self, command, reply=None, root=False, warn=''):
+
+    def execOnSelectedClients(self, command, reply=None,
+            mode=EM_SESSION_OR_SYSTEM, warn=''):
         clients = self.getSelectedClients()
         if len(clients) == 0: # No client selected, send the command to all
             clients = self.cstore
         else: # Show the warning only when no clients are selected
             warn = ''
-        self.execOnClients(command, clients, reply, root, warning=warn)
-    
-    # END_FIXUS
+        self.execOnClients(command, clients, reply, mode, warning=warn)
+
 
     def warnDlgPopup(self, warning):
         dlg = self.get('executionwarning')
@@ -852,7 +928,7 @@ which is incompatible with the current epoptes version.\
         else:
             return False
 
+
     def printErrors(self, error):
         print '  **Twisted error:', error
         return
-
